/*  pdtfetch.c       eej      8/93
        release 1.1  eej      4/94  added LITTLE_ENDIAN support

    reads PDATS traces for dineroIII simulator

    Modify dinero source as follows:
  
      1.  In main.c, insert a call to initPDATS
          after the existing call to init(...).

      2.  In fetch.c, (2 places)
          replace "readflag = sscanxx(linebuffer,labelp,addrp);"
          with "readflag = pdtfetch(labelp, addrp)"
	  Also comment out the preceding lines that use linebuffer

    You might also want to add the following to main.c to indicate the
    modification:

      printf("---PDATS input by Jiheng Ha and E.E. Johnson.\n");

*/

#include <stdio.h>

static unsigned long CurrentAddr[8];
static long addroff,tsoff;
static int type=0, NumRep;

static int count = 0;

initPDATS()
{
    int i;

    if(getchar() != 0xff) {
	fprintf(stderr, "pdtfetch: file not in PDATS format\n");
	exit(-1);
    }
    // fprintf(stderr, "pdtfetch: file is PDATS\n");
    for(i=0;i<=7;i++)
      CurrentAddr[i]=0;
    NumRep = 0;

    switch (getchar()) {
      case 3:         /* time stamps, w/repeats  */
      case 2:         /* time stamps, no repeats */
      case 1:         /* no time stamps, w/repeats  */
      case 0:         /* no time stamps, no repeats */
	break;
      default:
	fprintf(stderr, "pdtfetch: unrecognized format\n");
	exit(-2);
    }
    // fprintf(stderr, "pdtfetch: format OK\n");
}

/**************************************************************/

pdtfetch(labelp, addrp)
int *labelp, *addrp;
{
    register int c, tscode, addrcode, i;
    short o2;    /* assumed two bytes! */

    // fprintf(stderr, "pdtfetch: pdtfetch(%d)\n", count);

    if (NumRep--) {
	*labelp = type;
	*addrp = (CurrentAddr[type] += addroff);
	// fprintf(stderr, "Ref %6d\t%d\t%08x\n", ++count, type, *addrp);
	return 2;
    }
    if ((c=getchar()) != EOF) {
	type= c & 7;
	tscode = (c & 0x18) >>3;
	addrcode = (c & 0x60) >>5;
	if(c & 0x80)         /*** repeated ***/
	  NumRep = getchar();
	else
	  NumRep=1;

	switch (addrcode) {
	  case 0:
	    addroff=4;
	    break;
	  case 1:    /* one byte of offset */
	    if ((addroff = getchar()) & 0x80)
	      addroff |= 0xffffff80;
	    break;
	  case 2:    /* two bytes of offset */
#ifdef LITTLE_ENDIAN
	    if ((addroff = (getchar() << 8)) & 0x8000)
	      addroff |= 0xffff8000;
	    addroff |= getchar();
#else  /* BIG_ENDIAN */
	    fread((char *) &o2, 2, 1, stdin);
	    addroff = o2;
#endif
	    break;
	  case 3:    /* four bytes of offset */
#ifdef LITTLE_ENDIAN
	    addroff=0;
	    for(i=4;i>0;i--)
	      addroff |= getchar() << ((i-1) << 3);
#else  /* BIG_ENDIAN */
	    fread((char *) &addroff, 4, 1, stdin);
#endif
	    break;
	}

	if (tscode > 1) fread(&tsoff, (tscode-1), 1, stdin); /* throw away */

	*labelp = type;
	*addrp = (CurrentAddr[type] += addroff);
	NumRep--;
	// fprintf(stderr, "Ref %6d\t%d\t%08x\n", ++count, type, *addrp);
	return 2;
    }
    else return EOF;
}
